#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# yarely documentation build configuration file, created by
# sphinx-quickstart.
#
# This file is execfile()d with the current directory set to its
# containing dir.

# Standard libary imports
import sys
import os

# Third party imports
from sphinx.ext.autodoc import ClassDocumenter

# Insert the project root dir as the first element in the PYTHONPATH.
# This lets us ensure that the source package is imported, and that its
# version is used.
cwd = os.getcwd()
project_root = os.path.dirname(cwd)
sys.path.insert(0, project_root)

# Local imports
import yarely  # NOQA -- flake8 [ignore module level import not at top of file]


# -- General configuration ---------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named "sphinx.ext.*") or your custom ones.
extensions = ["sphinx.ext.autodoc", "sphinx.ext.viewcode"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# General information about the project.
project = u"Yarely"
author = yarely.__author__
copyright = yarely.__copyright__
version = yarely.__version__  # The short X.Y version.
release = yarely.__version__  # The full version, including alpha/beta/rc tags.

# Ignore the common prefix that all Yarely packages have
modindex_common_prefix = ["yarely."]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

autoclass_content = 'both'


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = "alabaster"

# Add any paths that contain custom static files (such as style sheets)
# here, relative to this directory. They are copied after the builtin
# static files, so a file named "default.css" will overwrite the builtin
# "default.css".
html_static_path = ["_static"]

# Output file base name for HTML help builder.
htmlhelp_basename = "yarelydoc"


# -- Options for LaTeX output ------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [(
  "index", "yarely.tex", u"Yarely Documentation", author, "manual"
)]


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [("index", "yarely", u"Yarely Documentation", [author], 1)]


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [(
  "index", "yarely", u"Yarely Documentation", author, "yarely",
  yarely.__shortdescription__, "Miscellaneous"),
]


# -- Autodoc functions -------------------------------------------------

def skip(app, what, name, obj, skip, options):
    # This ensures that the following private classes get documented:
    #   yarely.core.config.parse_config._YarelyConfig
    #   yarely.core.scheduling.contextconstraintsparser.\
    #     _ContextConstraintsParser
    #   yarely.core.scheduling.schedulers.lottery.duration_based_allocator.\
    #     _DurationBasedAllocator
    #   yarely.core.scheduling.schedulers.lottery.equal_distribution_allocator.\
    #     _EqualDistributionAllocator
    #   yarely.core.scheduling.schedulers.lottery.random_allocator.\
    #     _RandomAllocator
    #   yarely.core.scheduling.schedulers.lottery.ratio_allocator.\
    #     _RatioAllocator
    #   yarely.core.scheduling.schedulers.lottery.recency_based_allocator.\
    #     _RecencyBasedAllocator
    # (See my stackoverflow question here for an explaination of what I'm
    # trying to achieve here:
    #   http://stackoverflow.com/questions/38765577/overriding-sphinx-autodoc-alias-of-for-import-of-private-class
    # )
    # TODO:
    #    yarely.core.content.constants._ARG_RENDER_NAMESPACE
    #    yarely.core.content.constants._MIME_TYPE_CONFIG_MAP
    if (
        what == "module" and
        name in (
          "_ContextConstraintsParser",
          "_DurationBasedAllocator", "_EqualDistributionAllocator",
          "_RandomAllocator", "_RatioAllocator", "_RecencyBasedAllocator",
          "_YarelyConfig"
        ) 
    ):
        #options['undoc-members'] = False
        return False
    return skip

def setup(app):
    app.connect('autodoc-skip-member', skip)



# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.



# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.


""" Yarely module. """

__author__ = 'Lancaster University'
__copyright__ = "Copyright (C) 2011-2017 {author}".format(author=__author__)
__license__ = "Apache Software License 2.0"
__email__ = 'e-campus-help@lancaster.ac.uk'
__version__ = '0.1.0'
__shortdescription__ = (
    "Yarely is an open Digital Signage Platform developed by the eCampus team "
    "at Lancaster University."
)



# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.


# Standard library imports
import doctest
import unittest

# Local (Yarely) imports
from yarely.core.helpers import conversions


def load_tests(loader, tests, ignore):
    """FIXME

    :param loader: FIXME.
    :type loader: FIXME.
    :param tests: FIXME.
    :type tests: FIXME.
    :param ignore: FIXME.
    :type ignore: FIXME.
    :rtype: FIXME.

    """
    tests.addTests(doctest.DocTestSuite(conversions))
    return tests


class UnitOfInformationInBytesTestCase(unittest.TestCase):
    """FIXME"""

    def test_suffixless_number(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1")
        self.assertEqual(result, 1)

    def test_unparsable(self):
        """FIXME"""
        with self.assertRaises(conversions.UnitOfInformationConversionError):
            conversions.unit_of_information_in_bytes("unparsable")

    def test_1KB_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1KB")
        self.assertEqual(result, 1000)

    def test_1KIB_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1KIB")
        self.assertEqual(result, 1024)

    def test_1K_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1K")
        self.assertEqual(result, 1000)

    def test_1KI_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1KI")
        self.assertEqual(result, 1024)

    def test_1kb_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1kb")
        self.assertEqual(result, 1000)

    def test_1kib_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1kib")
        self.assertEqual(result, 1024)

    def test_1k_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1k")
        self.assertEqual(result, 1000)

    def test_1ki_value(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1ki")
        self.assertEqual(result, 1024)

    def test_in_str_spacing(self):
        """FIXME"""
        result = conversions.unit_of_information_in_bytes("1 KB")
        self.assertEqual(result, 1000)

    def test_1TB_value(self):
        """FIXME"""
        byte = 1
        kb = 1000 * byte
        mb = 1000 * kb
        gb = 1000 * mb
        tb = 1000 * gb

        result = conversions.unit_of_information_in_bytes("1TB")
        self.assertEqual(result, tb)

    def test_1TIB_value(self):
        """FIXME"""
        byte = 1
        kib = 1024 * byte
        mib = 1024 * kib
        gib = 1024 * mib
        tib = 1024 * gib

        result = conversions.unit_of_information_in_bytes("1TIB")
        self.assertEqual(result, tib)


class TimeIntervalInSecondsTestCase(unittest.TestCase):
    """FIXME"""

    def test_suffixless_number(self):
        """FIXME"""
        result = conversions.time_interval_in_seconds("1")
        self.assertEqual(result, 1)

    def test_unparsable(self):
        """FIXME"""
        with self.assertRaises(conversions.TimeIntervalConversionError):
            conversions.time_interval_in_seconds("unparsable")

    def test_1minute_value(self):
        """FIXME"""
        result = conversions.time_interval_in_seconds("1minute")
        self.assertEqual(result, 60)

    def test_1minutes_value(self):
        """FIXME"""
        result = conversions.time_interval_in_seconds("1minutes")
        self.assertEqual(result, 60)

    def test_1min_value(self):
        """FIXME"""
        result = conversions.time_interval_in_seconds("1min")
        self.assertEqual(result, 60)

    def test_1mins_value(self):
        """FIXME"""
        result = conversions.time_interval_in_seconds("1mins")
        self.assertEqual(result, 60)

    def test_in_str_spacing(self):
        """FIXME"""
        result = conversions.time_interval_in_seconds("60 seconds")
        self.assertEqual(result, 60)

    def test_2week_value(self):
        """FIXME"""
        second = 1
        minute = 60 * second
        hour = 60 * minute
        day = 24 * hour
        week = 7 * day

        result = conversions.time_interval_in_seconds("2 week")
        self.assertEqual(result, 2 * week)



# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.


# Standard library imports
import doctest
import unittest

# Local (Yarely) imports
from yarely.core.helpers import singleton, Singleton


def load_tests(loader, tests, ignore):
    """FIXME

    :param loader: FIXME.
    :type loader: FIXME.
    :param tests: FIXME.
    :type tests: FIXME.
    :param ignore: FIXME.
    :type ignore: FIXME.
    :rtype: FIXME.

    """
    tests.addTests(doctest.DocTestSuite(singleton))
    return tests


class SingletonTestCase(unittest.TestCase):
    """FIXME"""

    @classmethod
    def setUpClass(cls):
        """FIXME"""

        class SingletonA(object, metaclass=Singleton):
            def __init__(self, flag):
                self.flag = flag

        class SingletonB(object, metaclass=Singleton):
            def __init__(self, flag):
                self.flag = flag

        # singleton_a uses args, whilst singleton_b uses kwargs
        cls._singleton_a_true = SingletonA(True)
        cls._singleton_a_false = SingletonA(False)
        cls._singleton_b_true = SingletonB(flag=True)
        cls._singleton_b_false = SingletonB(flag=False)

    def test_single_instance_per_class(self):
        """FIXME"""
        self.assertIs(self._singleton_a_true, self._singleton_a_false)

    def test_different_classes_have_different_instances(self):
        """FIXME"""
        self.assertIsNot(self._singleton_a_true, self._singleton_b_true)

    def test_first_instance_args_taken(self):
        """FIXME"""
        self.assertIs(self._singleton_a_true.flag, True)

    def test_second_instance_args_ignored(self):
        """FIXME"""
        self.assertIs(self._singleton_a_false.flag, True)

    def test_first_instance_kwargs_taken(self):
        """FIXME"""
        self.assertIs(self._singleton_b_true.flag, True)

    def test_second_instance_kwargs_ignored(self):
        """FIXME"""
        self.assertIs(self._singleton_b_false.flag, True)



# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.


# Standard library imports
import datetime
import doctest
import random
import time
import unittest
from xml.etree import ElementTree

# Local (Yarely) imports
from yarely.core.helpers.base_classes import constraint


def load_tests(loader, tests, ignore):
    """FIXME

    :param loader: FIXME.
    :type loader: FIXME.
    :param tests: FIXME.
    :type tests: FIXME.
    :param ignore: FIXME.
    :type ignore: FIXME.
    :rtype: FIXME.

    """
    tests.addTests(doctest.DocTestSuite(constraint))
    return tests


class DateTimeConstraintConditionTestCase(unittest.TestCase):
    """FIXME"""

    def test_datetime(self):
        """FIXME"""
        now = datetime.datetime.today()
        result = constraint.DateTimeConstraintCondition(now)
        self.assertEqual(result.data, now)

    def test_float_timestamp(self):
        """FIXME"""
        now = time.time()
        datetime_now = datetime.datetime.fromtimestamp(now)
        result = constraint.DateTimeConstraintCondition(now)
        self.assertEqual(result.data, datetime_now)

    def test_int_timestamp(self):
        """FIXME"""
        now = int(time.time())
        datetime_now = datetime.datetime.fromtimestamp(now)
        result = constraint.DateTimeConstraintCondition(now)
        self.assertEqual(result.data, datetime_now)

    def test_list_ymdhms(self):
        """FIXME"""
        (year, month, day) = (2012, 1, 12)
        (hour, minute, second) = (14, 13, 50)
        result = constraint.DateTimeConstraintCondition(
            [year, month, day, hour, minute, second]
        )
        self.assertEqual(
            result.data,
            datetime.datetime(year, month, day, hour, minute, second)
        )

    def test_list_ymdhmsm(self):
        """FIXME"""
        (year, month, day) = (2012, 1, 12)
        (hour, minute, second, microsecond) = (14, 13, 50, 888)
        result = constraint.DateTimeConstraintCondition(
            [year, month, day, hour, minute, second, microsecond]
        )
        self.assertEqual(
            result.data, datetime.datetime(
                year, month, day, hour, minute, second, microsecond
            )
        )

    def test_list_ymdhmsmz(self):
        """FIXME"""
        (year, month, day) = (2012, 1, 12)
        (hour, minute, second, microsecond) = (14, 13, 50, 888)
        tzinfo = None
        result = constraint.DateTimeConstraintCondition(
            [year, month, day, hour, minute, second, microsecond, tzinfo]
        )
        self.assertEqual(
            result.data, datetime.datetime(
                year, month, day, hour, minute, second, microsecond, tzinfo
            )
        )

    def test_str_timestamp(self):
        """FIXME"""
        now = time.time()
        datetime_now = datetime.datetime.fromtimestamp(now)
        result = constraint.DateTimeConstraintCondition(str(now))
        self.assertEqual(result.data, datetime_now)

    def test_str_datetime(self):
        """FIXME"""
        datetimestr = '2012-01-12 14:13:50'
        result = constraint.DateTimeConstraintCondition(datetimestr)
        self.assertEqual(
            result.data, datetime.datetime.strptime(
                datetimestr, constraint.DateTimeConstraintCondition.STR_FORMAT
            )
        )

    def test_str_unparsable(self):
        """FIXME"""
        with self.assertRaises(constraint.DateTimeConstraintConditionError):
            constraint.DateTimeConstraintCondition('unparsable')

    def test_tuple_datetime(self):
        """FIXME"""
        today = datetime.date.today()
        noon = datetime.time(12)
        result = constraint.DateTimeConstraintCondition((today, noon))
        self.assertEqual(result.data, datetime.datetime.combine(today, noon))


class DayOfWeekConstraintConditionTestCase(unittest.TestCase):
    """FIXME"""

    def test_between(self):
        """Tests a valid day-of-week constraint defined with a between tag.

        Verifies that a day-of-week constraint containing a single
        'between' tag is correctly parsed into a DayOfWeekConstraint object.

        """
        between_test_str = '''
            <day-of-week>
                <between start="monday" end="sunday"/>
            </day-of-week>
        '''
        etree = ElementTree.XML(between_test_str)
        dow_constraint = constraint.DayOfWeekConstraint(etree)
        for i in range(7):
            self.assertEqual(
                dow_constraint._data[i].start_time, datetime.time.min
            )
            self.assertEqual(
                dow_constraint._data[i].end_time, datetime.time.max
            )

    def test_weekdays(self):
        """FIXME"""
        weekdays_test_str = '''
            <day-of-week>
                <Saturday time_start="09:00:00" time_end="12:00:00"/>
                <Sunday time_start="10:00:00" time_end="11:30:00"/>
            </day-of-week>
        '''
        etree = ElementTree.XML(weekdays_test_str)
        dow_constraint = constraint.DayOfWeekConstraint(etree)
        for i in range(5):
            self.assertIsNone(dow_constraint._data[i].start_time)
        self.assertEqual(
            dow_constraint._data[5].start_time, datetime.time(9, 0, 0)
        )
        self.assertEqual(
            dow_constraint._data[6].start_time, datetime.time(10, 0, 0)
        )
        self.assertEqual(
            dow_constraint._data[5].end_time, datetime.time(12, 0, 0)
        )
        self.assertEqual(
            dow_constraint._data[6].end_time, datetime.time(11, 30, 0)
        )


class PriorityConstraintConditionTestCase(unittest.TestCase):
    """FIXME"""

    def test_str(self):
        priorities = constraint.PriorityConstraint.ALL_PRIORITIES
        index = random.randrange(len(priorities) - 1)
        result = constraint.PriorityConstraintCondition(priorities[index])
        self.assertEqual(result.data, priorities[index])

    def test_str_unparsable(self):
        """FIXME"""
        with self.assertRaises(constraint.PriorityConstraintConditionError):
            constraint.PriorityConstraintCondition('unparsable')

    def test_index(self):
        """FIXME"""
        priorities = constraint.PriorityConstraint.ALL_PRIORITIES
        index = random.randrange(len(priorities) - 1)
        result = constraint.PriorityConstraintCondition(index)
        self.assertEqual(result.data, priorities[index])

    def test_index_outofrange(self):
        """FIXME"""
        priorities = constraint.PriorityConstraint.ALL_PRIORITIES
        with self.assertRaises(constraint.PriorityConstraintConditionError):
            constraint.PriorityConstraintCondition(len(priorities))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.


# Standard library imports
from setuptools import setup

# Local imports
import yarely


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

platforms = ["Mac OS X >= 10.6"]
requirements = ['pyserial', 'tornado', 'zmq', 'PyObjC', 'certifi']
test_requirements = []

setup(
    author=yarely.__author__,
    author_email=yarely.__email__,

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=sorted([
        "Development Status :: 4 - Beta",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "Natural Language :: English",

        # There is no specific classifier for Apache 2.0 right now (Aug 2016)
        # If there were, we could build this using yarely.__license__.
        "License :: OSI Approved :: Apache Software License",

        # OS support -- currently OS X >= 10.6
        # See also the platforms variable (no format defined by setuptools).
        "Environment :: MacOS X :: Cocoa",
        "Operating System :: MacOS :: MacOS X",

        # Supported Python versions
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6"
    ]),

    description=yarely.__shortdescription__,
    include_package_data=True,
    install_requires=requirements,
    keywords="yarely digital signage",
    license=yarely.__license__,
    long_description="{readme}\n\n{changelog}".format(
      readme=readme, changelog=history
    ),
    name="yarely",
    packages=["yarely"],
    platforms=platforms,
    scripts=[],
    tests_require=test_requirements,
    url="https://github.com/opendisplays/yarely",
    version=yarely.__version__,
)



# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.


# Standard library imports
# import doctest

# Local (Yarely) imports
from yarely.core import displaycontroller  # NOQA


def load_tests(loader, tests, ignore):
    """FIXME

    :param loader: FIXME.
    :type loader: FIXME.
    :param tests: FIXME.
    :type tests: FIXME.
    :param ignore: FIXME.
    :type ignore: FIXME.
    :rtype: FIXME.

    """
    # tests.addTests(doctest.DocTestSuite(displaycontroller))
    return tests



# -*- coding: utf-8 -*-
#
# Copyright 2011-2016 Lancaster University.
#
#
# This file is part of Yarely.
#
# Licensed under the Apache License, Version 2.0.
# For full licensing information see /LICENSE.


# Standard library imports
import doctest
import tempfile
import unittest
import urllib.parse

# Local (Yarely) imports
from yarely.core import platform


def load_tests(loader, tests, ignore):
    """FIXME

    :param loader: FIXME.
    :type loader: FIXME.
    :param tests: FIXME.
    :type tests: FIXME.
    :param ignore: FIXME.
    :type ignore: FIXME.
    :rtype: FIXME.

    """
    tests.addTests(doctest.DocTestSuite(platform))
    return tests


class GetAvailableSpaceInBytesTestCase(unittest.TestCase):
    """FIXME"""

    @classmethod
    def setUpClass(cls):
        cls._path = tempfile.gettempdir()

    def setUp(self):
        self._result = platform.get_available_space_in_bytes(self._path)

    def test_numerical_result(self):
        """FIXME"""
        self.assertIsInstance(self._result, int)

    def test_positive_result(self):
        """FIXME"""
        self.assertGreaterEqual(self._result, 0)


class URILocalPathConversionTestCase(unittest.TestCase):
    """FIXME"""

    @classmethod
    def setUpClass(cls):
        cls._path = tempfile.gettempdir()

    def setUp(self):
        self._result = platform.get_uri_from_local_path(self._path)

    def test_geturi_string_result(self):
        """FIXME"""
        self.assertIsInstance(self._result, str)

    def test_geturi_reverse_result(self):
        """FIXME"""
        self.assertEqual(
            self._path, platform.get_local_path_from_uri(self._result)
        )

    def test_getlocalpath_raises(self):
        """FIXME"""
        erroneous_result = urllib.parse.ParseResult(
            scheme='http', netloc='', path=self._path, params='', query='',
            fragment=''
        )
        parsed_erroneous = urllib.parse.urlunparse(erroneous_result)
        with self.assertRaises(platform.PlatformError):
            platform.get_local_path_from_uri(parsed_erroneous)
